/********************************************************************
 * FILE: metameme.c
 * AUTHOR: William Stafford Noble
 * CREATE DATE: 05-19-98
 * PROJECT: MHMM
 * COPYRIGHT: 2001-2008, WSN
 * DESCRIPTION: Global functions for the Meta-MEME toolkit.
 ********************************************************************/
#include <stdio.h>
#include <time.h>
#include "utils.h"
#include "metameme.h"

/*************************************************************************
 * Print the program header, including help info, if necessary.
 *************************************************************************/
void write_header
  (HEADER_T program,     // Which program called this function? */
   char*    description, // Text description.
   char*    meme_file,   // Motif filename.
   char*    model_file,  // HMM filename.
   char*    seq_file,    // Name of the input sequence file.
   FILE*    outfile)     // Stream to write to.
{
  time_t current_time;
  const int head_skip = 18; // Skip SVN keyword
  const int tail_skip = 2; // Skip close of SVN keyword
  const char *release_date = ARCHIVE_DATE + head_skip;

  fprintf(outfile, "# Meta-MEME - Motif-based hidden Markov modeling toolkit.\n");
  fprintf(outfile, "# \n");
  int i = strlen(release_date) - tail_skip;
  fprintf(outfile, "# version %s (Release date: %.*s)\n", VERSION,
	  i, release_date);
  fprintf(outfile, "# \n");
  switch (program) {
  case MHMM_HEADER :
    fprintf(outfile, "# Program: mhmm\n");
    fprintf(outfile, "# File contents: Motif-based hidden Markov model\n");
    break;
  case MHMMT_HEADER :
    fprintf(outfile, "# Program: mhmmt\n");
    fprintf(outfile, "# File contents: Trained hidden Markov model\n");
    break;
  case MHMMS_HEADER :
    fprintf(outfile, "# Program: mhmms\n");
    fprintf(outfile, "# File contents: Database search results\n");
    break;
  case MHMMSCAN_HEADER :
    fprintf(outfile, "# Program: mhmmscan\n");
    fprintf(outfile, "# File contents: Database search results\n");
    break;
  case MHMMA_HEADER:
    fprintf(outfile, "# Program: mhmma\n");
    fprintf(outfile, "# File contents: Motif-based multiple alignment\n");
    break;
  case MHMMF_HEADER:
    fprintf(outfile, "# Program: mhmmf\n");
    fprintf(outfile, "# File contents: Gradient matrix from a hidden Markov model and a set of sequences\n");
    break;
  case INVALID_HEADER:
    die("Illegal header type.\n");
  }
  if (description != NULL) {
    fprintf(outfile, "# Model description: %s\n", description);
  }
  if (meme_file != NULL) {
    fprintf(outfile, "# Motif file: %s\n", meme_file);
  }
  if (model_file != NULL) {
    fprintf(outfile, "# HMM file: %s\n", model_file);
  }
  if (seq_file != NULL) {
    switch (program) {
    case MHMMT_HEADER :
    case MHMMA_HEADER:
    case MHMMF_HEADER:
      fprintf(outfile, "# Sequence file: %s\n", seq_file);
      break;
    case MHMMS_HEADER :
    case MHMMSCAN_HEADER :
      fprintf(outfile, "# Database file: %s\n", seq_file);
      break;
    case MHMM_HEADER :
      die("Sequence file given to mhmm.\n");
      break;
    case INVALID_HEADER:
      die("Illegal header type.\n");
    }
  }
  current_time = time(0);
  fprintf(outfile, "# Create date: %s", ctime(&current_time));
  fprintf(outfile, "# \n");
  fprintf(outfile, "# For further information on how to interpret these ");
  fprintf(outfile, "results or to get a copy\n");
  fprintf(outfile, "# of the Meta-MEME software, ");
  fprintf(outfile, "please access http://metameme.sdsc.edu.\n");
  fprintf(outfile, "# \n");
  fprintf(outfile, "# If you use this program in your research, please cite\n");
  fprintf(outfile, "# \n");
  fprintf(outfile, "#  William N. Grundy, Timothy L. Bailey, Charles P. ");
  fprintf(outfile, "Elkan and Michael E.\n");
  fprintf(outfile, "#   Baker.  \"Meta-MEME: Motif-based hidden ");
  fprintf(outfile, "Markov models of protein\n");
  fprintf(outfile, "#   families.\" Computer Applications ");
  fprintf(outfile, "in the Biosciences.  13(4):397-406, 1997.\n");
  fprintf(outfile, "# \n");

  fprintf(outfile, "# *****************************************************************************\n");
}


