#!@WHICHPERL@ -w

#
# Output the documentation on supported databases.
# 

#use strict;
#use warnings;
use CGI;
use CGI::Carp qw(fatalsToBrowser set_message);

my $dir = "@MEME_DIR@";    # installed directory
my $dbnames =  "$dir/etc/mast_db_names.csv";

my $cgi = new CGI;

# Open list of supported databases.
my $result = open(DB_NAMES, "<$dir/etc/mast_db_names.csv");

if (not $result) {
  print $cgi->header(-status=>'500 Internal Server Error');
  die "Unable to open the list of supported databases.\n";
}
else {
  # read in the file
  my @lines = <DB_NAMES>; 
  close(DB_NAMES);
 
  # get the number of separator lines
  my @categories = ();
  for $_ (@lines) {
    next if (/^#/ || !/\S/);		# skip comments and blank lines
    my @fields = split(/,/);
    if ($fields[0] !~ /\S/ && $fields[4] =~ /\S/) {
      $fields[4] =~ s/^-+//g; $fields[4] =~ s/-+$//g;
      push(@categories, $fields[4]) 
    }
  } 

  my $form = "<html>\n";
  $form .= make_header();
  $form .= start_body(@categories);
  $form .= add_dbs(@lines);
  $form .= finish_body();
  $form .= "</html>";

  print "Content-type: text/html\n\n$form";
}

sub make_header {
  my $content = qq %
  <head>
    <title>Databases available for search </title>
    <!--- provide relative path to html directory (parent of cgi-bin) --->
    <script language="javascript" type="text/javascript">var html_path = "../"</script>
    <script src="../template-css.js" type="text/javascript"></script> 
    <script src="../template-header.js" type="text/javascript"></script> 
  </head>
  %; 	# end of quote

  return($content);
} # make_header

sub start_body {
  my (@categories) = @_;

  # create list of categories
  my ($cat, $i, $cat_descr);
  $i = 1;
  foreach $cat (@categories) {
    $cat_descr .= "<li><h3><a href=#cat$i>$cat</a></h3>\n";
    $i++;
  }
  my $ncats = @categories;

  my $content = qq %
   <body class="body">
    <table class="maintable">
      <tr>
        <td class="maintablewidth">
          <div id="main">
            <script src="../meme-suite-logo.js" type="text/javascript"></script>
            <noscript>
              <h1>MEME Suite</h1>
              The MEME Suite web application requires the use of JavaScript
              <br />
              Javascript doesn't seem to be available on your browser.
            </noscript>
            <hr />
            <h1 align="center">Sequence databases available for search</h1>

            <hr />
            <h3>The sequence databases that can be searched are grouped into
            $ncats categories:</h3>
            <ul> 
              $cat_descr
            </ul>
  %; 	# end of quote
} # start_menu

sub add_dbs {
  my (@lines) = @_;

  my $i = 1;
  my $content;

  foreach $_ (@lines) {
    next if (/^#/ || !/\S/);		# skip comments and blank lines
    my @fields = split(/,/);
    if ($fields[0] !~ /\S/) {
      $fields[4] =~ s/^-+//g; $fields[4] =~ s/-+$//g;
      $content .= qq %
	    <a name="cat$i"></a>
	    <hr />
	    <h3>$fields[4]</h3>
	    <hr />
      % ; 		# end of quote
      $i++;
    } else {
      $content .= qq %
	    <dl>
	      <dt><b>$fields[4]</b></dt>
	      <dd>$fields[5]</dd>
	      <dd>(protein: $fields[1] nucleotide: $fields[2] short: $fields[3])</dd>
            </dl>
        % ; 		# end of quote
    }
  }
  $content .= qq %
	    <hr />
    % ;		# end of quote


  return($content);
} # add_dbs

sub finish_body {
  my $content .= qq %
            <script src="../template-footer.js" type="text/javascript"></script>
          </div>
        </td>

      </tr>
    </table>
   </body>
    % ; # end of quote

  return($content);
} # finish_body
