#!@WHICHPERL@
##
## $Id: process_request.pl 2970 2008-08-04 01:39:57Z tbailey $
##
## $Log: process_request.pl,v $
## Revision 1.6.6.1  2006/02/16 23:22:35  nadya
## update path to show the background image on the page.
##
## Revision 1.6  2005/12/01 01:18:53  tbailey
## Add handling of JASPAR searches with MEME motifs.
##
## Revision 1.5  2005/10/02 01:00:10  nadya
## move meme-client and mast-client names into Globals and use variables instead.
##
## Revision 1.4  2005/09/16 01:00:40  nadya
## add csh extention to capture output for the web page in process-request.cgi
##
## Revision 1.3  2005/09/13 18:40:01  nadya
## rm "web" from url string
##
## Revision 1.2  2005/08/07 05:58:26  nadya
## use vairables in Globals for meme locations,
## fix locatio of website.
##
## Revision 1.1.1.1  2005/07/25 23:27:26  nadya
## Importing from meme-3.0.14, and adding configure/make
##
##

use lib qw(@PERLLIBDIR@);
$dir = "@MEME_DIR@";		# installed directory
$bin_dir = "$dir/bin";		# directory for executables
$logs_dir = "$dir/logs";	# directory for executables

#
# Process a request submitted from a MEME results form.
# Requests include:
#	Submit to MAST
#	View hidden field
#	Submit to BLOCKS
#	Submit to Meta-MEME
#	Submit to Tomtom
#	Print a man page
#
#
# Changes:
# print header ( Content-type:  text/html\n\n in the beginning of the script )
# this fixes the 'Error 500 malformed header from script' error
# 12/12/2002	JKP
# 03/24/2003	TLB; fixed problems with all types of submissions
#		
#
# USER SETTABLE LINKS
#
################################################################################
#
# You can change this if you wish to use a different BLOCKS server
$blocks_url = "http://blocks.fhcrc.org/blocks-bin/process_blocks.pl";
#
# You can change this to use a different GLAM2 server
$mast_url = "@SITE_URL@/cgi-bin/mast.cgi";
#
# You can change this if you wish to use a different TOMTOM server
$tomtom_url = "@SITE_URL@/cgi-bin/tomtom.cgi";
#
# You can change this to use a different GLAM2 server
$glam2_url = "@SITE_URL@/cgi-bin/glam2.cgi";
#
# You can change this to use a different GLAM2SCAN server
$glam2scan_url = "@SITE_URL@/cgi-bin/glam2scan.cgi";
#
# You can change this if you wish to use a different Meta-MEME server
#$metameme_url = "http://metameme.sdsc.edu/cgi-bin/mhmm_process_request.cgi";
$metameme_url = "@SITE_URL@/cgi-bin/mhmm_process_request.cgi";
#
################################################################################

# requires
# use the CGI package
use CGI qw/:standard/;			# use the CGI package
use HTTP::Request::Common qw(POST);
use LWP::UserAgent;

#
# get the action (type of submit button pressed) and branch on it
#
$action = param('action');			# which submit button pressed
$nmotifs = param('nmotifs');			# number of motifs
#print header;

if ($action eq "MAST") {
  mast_search(1, $nmotifs);
} elsif ($action =~ /^BLOCKS/) {
  submit_block(1..$nmotifs);
} elsif ($action =~ /^View (\w+) (\d+)/) {
  hidden_field($1, $2);
} elsif ($action =~ /^Submit BLOCK (\d+)/) {
  submit_block($1);
} elsif ($action =~ /^Scan PSSM (\d+)/) {
  mast_search($1, $1);
} elsif ($action =~ /^Compare PSPM (\d+)/) {
  submit_pspm_to_tomtom($1);
} elsif ($action =~ /^COMPARE PSPM (\d+)/) {
  submit_pspm_to_tomtom($1);
} elsif ($action =~ /^View motif summary/) {
  hidden_field('motif-summary');
} elsif ($action =~ /MetaMEME/){
  forward_query("$metameme_url");
} elsif ($action =~ /re-run GLAM2/){
  forward_query("$glam2_url")
} elsif ($action =~ /^Scan alignment (\d+)/) {
  $params{'inline_name'} = "alignment_$1";
  $params{'inline_aln'} = param('aln'.$1);
  forward_query("$glam2scan_url")
} elsif ($action =~ /MEME Man Page/){
  print_man("meme");
} elsif ($action =~ /MAST Man Page/){
  print_man("mast");
} else {
  print_header($action);			# print a response header
  print "action: $action\n";
  print "<H1>Feature not implemented yet.</H1>\n";
  exit(1);
}

# all done!
exit(0);

#
# forward a query to another service
#
sub forward_query {
  my($cgi) = @_;
  print "Content-type: text/html\n\n";
  # Get the contents of the query
  foreach $pname (param()) { $params{$pname} = param($pname); }
  undef $params{'action'};
  # post the query
  $ua = LWP::UserAgent->new();
  my $req = POST "$cgi", [%params];
  my $request = $ua->request($req);
  my $content = $request->content;
  print $content;
} # forward_query

#
# Submit a MAST query
#
sub mast_search {
  my($start, $stop) = @_;
  my(%params);

  print "Content-type: text/html\n\n";		# start form

  # check input
  if ($stop - $start < 0) {
    print_header("MAST Search");
      print "<html><body><PRE>There are no valid motifs in your MEME file.</PRE></body></html>\n";
    exit(1);
  }

  # set up the parameters to pass to MAST
  $params{'alphabet'} = param('alphabet');
  $params{'inline_name'} = param('name');
  $params{'inline_motifs'} = "ALPHABET= " . param('alphabet') . "\n";
  for ($i=$start; $i<=$stop; $i++) { 
    $params{'inline_motifs'} .= param('pssm'.$i); 
  }
  $params{'address'} = $params{'address_verify'} = param('address');
  
  # post the query
  my $ua = LWP::UserAgent->new();
  my $req = POST "$mast_url", [%params];
  my $request = $ua->request($req);
  my $content = $request->content;
  print $content;
} # mast_search 

#
# view a hidden field
#
sub hidden_field {
  my($name, $number) = @_;

  print_header($action);
  if ($name eq "BLOCK") {
    $field = param('BLOCKS'.$number);		# get BLOCK
  } elsif ($name eq "FASTA") {
    $field = param('BLOCKS'.$number);
    $field = block2fasta($field);		# convert to FASTA
  } elsif ($name eq "RAW") {
    $field = param('BLOCKS'.$number);
    $field = block2raw($field);			# convert to raw format 
  } elsif ($name eq "PSSM") {
    $field = param('pssm'.$number);		# get PSSM
  } elsif ($name eq "PSPM") {
    $field = param('pspm'.$number);		# get PSPM
  } elsif ($name eq "alignment") {
    $field = param('aln'.$number);		# get PSPM
  } elsif ($name eq "motif-summary") {
    $field = param('motif-summary');		# get motif-summary
  } else { 
    print "Unknown hidden field type: $name\n";
    exit(1)
  }
  print "<PRE>$field</PRE>";
} # hidden_field

#
# Convert a BLOCK to RAW sequence format
#
sub block2raw {
  my($block) = @_;
  my($i, @lines, @words, $raw);

  $fasta = "";					# return value
  @lines = split(/\n/, $block); 		# split block into lines
  for ($i = 2; $i<$#lines; $i++) {
    last if $lines[$i] =~ /^\/\//;
    @words = split(/\s+/, $lines[$i]);		# split line into words
    # get sequence line
    $raw .= "$words[3]\n";
  }
  $raw;
} # block2raw

#
# Convert a BLOCK to FASTA
#
sub block2fasta {
  my($block) = @_;
  my($i, @lines, @words, $fasta, $start);

  $fasta = "";					# return value
  @lines = split(/\n/, $block); 		# split block into lines
  for ($i = 2; $i<$#lines; $i++) {
    last if $lines[$i] =~ /^\/\//;
    @words = split(/\s+/, $lines[$i]);		# split line into words
    # get id line and sequence line
    $start = substr($words[2], 0, length($words[2])-1);
    $fasta .= ">$words[0] ( start= $start )\n$words[3]\n";
  }
  $fasta;
} # block2fasta

#
# Submit a block to the blocks processor
#
sub submit_block {
  my(@numbers) = @_;
  my($blocks);

  print_header("Submit BLOCKS");
 
  # get the BLOCK(S)
  $blocks = "";
  foreach $number (@numbers) {
    $blocks .= param('BLOCKS'.$number);
  }

  $ua = LWP::UserAgent->new();
  my $req = POST $blocks_url, [ sequences => $blocks ];
  my $request = $ua->request($req);
  #$content = $ua->request($req)->as_string;
  $content = $request->content;
  # put in the absolute url's : this is FRAGILE!
  $content =~ s#HREF=\"#HREF=\"http://blocks.fhcrc.org#g;
  $content =~ s#ACTION=\"/blocks-bin#ACTION=\"http://blocks.fhcrc.org/blocks-bin#g;
  print $content;
} # submit_block

#
# Compare a PSPM to the TOMTOM motif database search tool
#
sub submit_pspm_to_tomtom {
  my($number) = @_;
  my($pspm, @fields, $i, $n, $w, $row, $col);

  print "Content-type: text/html\n\n";

  # get the motif PSPM 
  $_ = param('pspm'.$number);
  @fields = split;

  # ignore 1st 10 entries
  $nsites = $fields[7];               # save number of sites
  for ($i=0; $i<=$#fields-10; $i++) { $fields[$i] = $fields[$i+10] * $nsites; }
  $#fields = $#fields - 10;

  # rotate PSPM 90 degrees (natural format) as a string with newlines
  $n = $#fields + 1;		# number of entries in motif
  $w = $n/4;			# motif width
  $pspm = "";
  for ($row=0; $row<4; $row++) {
    for ($col=0; $col<$w; $col++) {
      $pspm .= " " . $fields[($col*4) + $row];
    }
    $pspm .= "\n";		# terminate row with newline
  }

  # create the request
  $ua = LWP::UserAgent->new();
  my $req = POST "$tomtom_url",
    Content_Type => 'multipart/form-data',
    Content => [ 'query' => $pspm ];
  my $request = $ua->request($req);
  $content = $request->content;

  # display the page
  print $content;
} # submit_pspm_to_tomtom


#
# Print a man page
#
sub print_man {
  my($command) = @_;
  print <<END; 
  Content-type: text/plain

END
  chdir("$logs_dir");
  $bin = "$bin_dir";
  @tmp = `$bin/$command`;
  print @tmp[3..$#tmp];
}

#
# start a response form
#
sub print_header {
  my($action) = @_;
  print <<END; 
Content-type: text/html

<HTML>
<TITLE> MEME SUITE - $action</TITLE>
<BODY BACKGROUND=\"../images/bkg.jpg\">
END
} # print_header
